<?php

namespace Drupal\menu_svg_icons\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Form controller for icon set forms.
 */
class IconSetForm extends EntityForm {

  /**
   * Constructs an IconSetForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager) {
    $this->entityQuery = $entity_type_manager->getStorage('menu_svg_icons_icon_set')->getQuery();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $icon_set = $this->entity;

    $form['#title'] = $this->operation == 'add' ? $this->t('Add icon set')
        :
        $this->t('Edit %label icon set', ['%label' => $icon_set->label()]);

    $form['label'] = [
      '#title' => $this->t('Name'),
      '#type' => 'textfield',
      '#default_value' => $icon_set->label,
      '#description' => $this->t('The human-readable name of this icon set. This name must be unique.'),
      '#required' => TRUE,
      '#size' => 30,
    ];

    $form['id'] = [
      '#type' => 'machine_name',
      '#default_value' => $icon_set->id(),
      '#machine_name' => [
        'exists' => [$this, 'exist'],
      ],
      '#disabled' => !$icon_set->isNew(),
      '#description' => $this->t('A unique machine-readable name for this icon set. It must only contain lowercase letters, numbers, and underscores.'),
    ];

    $form['description'] = [
      '#title' => $this->t('Description'),
      '#type' => 'textarea',
      '#default_value' => $icon_set->description,
      '#description' => $this->t('Describe this icon set.'),
    ];

    $form['placement'] = [
      '#title' => $this->t('Icon placement'),
      '#type' => 'radios',
      '#options' => [
        'left' => $this->t('Left'),
        'right' => $this->t('Right'),
      ],
      '#default_value' => ($icon_set->placement ? $icon_set->placement : 'left'),
      '#description' => $this->t('Define the icon placement.'),
    ];

    $form['source'] = [
      '#title' => $this->t('Icon source'),
      '#type' => 'textarea',
      '#rows' => 15,
      '#default_value' => $icon_set->source,
      '#description' => $this->t('Paste your svg sprite code'),
      '#required' => TRUE,
    ];

    $form['icon_height'] = [
      '#title' => $this->t('Icon height'),
      '#type' => 'number',
      '#default_value' => ($icon_set->icon_height ? $icon_set->icon_height : ''),
      '#description' => $this->t('Define height on icons. If no value is set, fallback to css rules'),
    ];

    $form['icon_width'] = [
      '#title' => $this->t('Icon width'),
      '#type' => 'number',
      '#default_value' => ($icon_set->icon_width ? $icon_set->icon_width : ''),
      '#description' => $this->t('Define height on icons. If no value is set, fallback to css rules'),
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    $source = $form_state->getValue('source');

    // @See: http://php.net/manual/en/function.libxml-use-internal-errors.php
    libxml_use_internal_errors(TRUE);
    $dom = simplexml_load_string($source);
    // Array with LibXMLError objects if there are any errors in the buffer, or
    // an empty array otherwise.
    $svg_errors = libxml_get_errors();

    if ($dom === FALSE || $svg_errors) {
      $form_state->setErrorByName('source',
        $this->t('The value of source has to be a valid xml string.')
      );

      // Iterate over errors generated by an invalid svg source.
      foreach ($svg_errors as $svg_error) {
        $message = $this->t('%message [line %line, column %column]', [
          '%message' => $svg_error->message,
          '%line' => $svg_error->line,
          '%column' => $svg_error->column,
        ]);

        \Drupal::messenger()->addMessage($message, 'error');
      }
    }
    else {
      // Check if the current SVG contain at least one valid symbol definition.
      $symbols = $dom->defs->symbol;
      if (!count($symbols)) {
        $form_state->setErrorByName('source',
          $this->t('The source have to contain at least one valid symbol.
            Check if there are any valid "symbol" tags inside the "defs" tag.')
        );
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function save(array $form, FormStateInterface $form_state) {
    $icon_set = $this->entity;
    $status = $icon_set->save();

    if ($status) {
      \Drupal::messenger()->addMessage($this->t('Saved the %label Icon set.', [
        '%label' => $icon_set->label(),
      ]));
    }
    else {
      \Drupal::messenger()->addMessage($this->t('The %label Icon set was not saved.', [
        '%label' => $icon_set->label(),
      ]), 'error');
    }

    $form_state->setRedirect('entity.menu_svg_icons_icon_set.collection');
  }

  /**
   * Helper function to check whether an IconSet configuration entity exists.
   */
  public function exist($id) {
    $entity = $this->entityQuery
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }

}
